package DE_EPAGES::Object::t::LastUpdate;
use base DE_EPAGES::Database::API::Test;
use strict;
use Test::More tests => 22;
use DateTime;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw( AddDuration );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadObjectByPath LoadRootObject ExistsObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );

use constant PROFILE_PACKAGE  => 'DE_EPAGES::Object::API::Object::LastUpdateProfile';
use constant PROFILE_NAME_1   => 'test-01';
use constant PROFILE_CLASS_1  => 'Object';
use constant PROFILE_XML_FILE => 'LastUpdateImports.xml';
use constant PROFILE_NAME_2   => 'test-02';
use constant PROFILE_CLASS_2  => 'Site';

use constant PROFILE_ALIAS => 'obj-profile';
use constant OBJECT_ALIAS_1   => 'obj-01';
use constant OBJECT_ALIAS_2   => 'obj-02';
use constant OBJECT_ALIAS_3   => 'obj-03';
use constant OBJECT_ALIAS_4   => 'obj-04';
use constant TEST_ALIAS => 'Test';
use constant TEST_PATH => '/'.TEST_ALIAS;

sub init {
    #create an object last update profile
    _InsertProfile( PROFILE_ALIAS, 'UnitOfMeasurement' );

    #create test objects
    my $TestFolder = LoadRootObject()->folder(TEST_ALIAS);

    _InsertTestObject_UOM( OBJECT_ALIAS_1, $TestFolder ); #test set lastupdate
    _InsertTestObject_UOM( OBJECT_ALIAS_2, $TestFolder ); #test another set last update
    _InsertTestObject_UOM( OBJECT_ALIAS_3, $TestFolder ); #test find last updates
    _InsertTestObject( OBJECT_ALIAS_4, PROFILE_CLASS_2, $TestFolder ); #test wrong class
}

sub cleanup {
    #remove object last update profile
    if( ExistsObjectByPath('/Classes/UnitOfMeasurement/LastUpdateProfiles/'.PROFILE_ALIAS) ) { ## no critic qw(ProhibitPathConcatenation)
        _GetProfile( PROFILE_ALIAS, 'UnitOfMeasurement' )->delete
    };

    if( ExistsObjectByPath(TEST_PATH) ) {
        LoadObjectByPath(TEST_PATH)->delete
    }
}

sub test {
    my $self = shift;
    #test object creation, getting info and deletion of profiles
    TestProfileObject( PROFILE_NAME_1, PROFILE_CLASS_1 );
    TestProfileXML( PROFILE_NAME_2, PROFILE_CLASS_2, PROFILE_XML_FILE );

    #test to set and get timestamps on specific objects
    my $dbi = $self->dbi;
    my $TestFolder = LoadObjectByPath(TEST_PATH);
    my $TestProfile = _GetProfile( PROFILE_ALIAS, 'UnitOfMeasurement' );
    my $Site = $TestProfile->getSite();
    my $LastTime = $dbi->currentDateTime;

    my %TimeStamp; #remember update times
    my $Alias = OBJECT_ALIAS_1;
    my $Object = $TestFolder->child( $Alias );
    $Object->setUpdated( $TestProfile );
    $TimeStamp{$Alias} = $dbi->currentDateTime;
    ok( $TestProfile->getLastUpdate( $Object ), 'get last update of object');

    $Alias = OBJECT_ALIAS_2;
    $Object = $TestFolder->child( $Alias );
    $Object->setUpdated( $TestProfile );
    $TimeStamp{$Alias} = $dbi->currentDateTime;
    ok( $TestProfile->getLastUpdate( $Object ), 'get last update of object');

    #test wrong class
    $Object = $TestFolder->child( OBJECT_ALIAS_4 );
    eval { $Object->setUpdated( $TestProfile ) };
    ok( ExistsError(), 'set last update with invalid class' );
    is( GetError()->code, 'NO_VALID_CLASS', 'set last update with invalid class' );

    #test search
    my $Iterator = $Site->getUpdatedObjects( $TestProfile, $LastTime );
    is( $Iterator->count, 2, 'find 2 updates' );
    while ( my($hUpdate) = $Iterator->next ) {
        my $Last = $hUpdate->{LastUpdatedOn};
        my $Obj = $hUpdate->{Object};
        my $ObjAlias = $Obj->alias;
        my $Expect = $TimeStamp{$ObjAlias};

        ok( abs($Last->epoch - $Expect->epoch)<1, "$ObjAlias $Last ok");
    };

    $Site->clearUpdatedObjects( $TestProfile, AddDuration($LastTime, 'seconds', 2) );
    $Iterator = $Site->getUpdatedObjects( $TestProfile, $LastTime );
    is( $Iterator->count, 0, 'clearUpdatedObjects' );
}


sub TestProfileObject {
    my ( $ProfileName, $ProfileClass ) = @_;
    my $Profile = _InsertProfile( $ProfileName, $ProfileClass );
    ok( $Profile->id, $Profile->pathString.' created by insert' );
    ok( _CheckProfile( $Profile, $ProfileName, $ProfileClass ), 'profile values ok' );
    ok( _DeleteProfile( $Profile ), 'delete profile');
}

sub TestProfileXML {
    my ( $ProfileName, $ProfileClass, $ProfileXML ) = @_;
    my $Profile = _InsertXMLProfile( $ProfileName, $ProfileClass, $ProfileXML );
    ok( $Profile->id, $Profile->pathString.' created by xml import' );
    ok( _CheckProfile( $Profile, $ProfileName, $ProfileClass ), 'profile values ok' );
    ok( _DeleteProfile( $Profile ), 'delete profile');
}

# insert a profile objec by insertObject method
sub _InsertProfile {
    my ( $ProfileName, $RefClassAlias ) = @_;
    my $ClassRef  = LoadClassByAlias( $RefClassAlias );
    my $ClassProfile = LoadClassByAlias( 'LastUpdateProfile' );
    my $Parent = $ClassRef->folder('LastUpdateProfiles');

    return $ClassProfile->insertObject({
        Parent        => $Parent,
        Alias         => $ProfileName,
        ProfileClass  => $ClassRef,
    });
}

# insert a profile objec by XML import
sub _InsertXMLProfile {
    my ( $Name, $Class, $XMLFile ) = @_;
    ImportObjects( $XMLFile );
    return LoadObjectByPath('/Classes')->child($Class)->child('LastUpdateProfiles')->child($Name);
}

sub _GetProfile {
    my ( $ProfileName, $RefClassAlias ) = @_;
    my $Class  = LoadClassByAlias( $RefClassAlias );
    return $Class->lastUpdateProfile($ProfileName);
}

sub _CheckProfile {
    my ( $Profile, $Name, $Class ) = @_;

    #check if profiles created to right package
    isa_ok( $Profile, PROFILE_PACKAGE, 'isa Profile ok' );

    #check profile class
    is( $Profile->className, 'LastUpdateProfile', 'Profile self class' );

    #compare values
    is( $Profile->alias, $Name, 'get Profile.Name' );
    is( $Profile->get('ProfileClass')->alias, $Class, 'get Profile.Class' );

    return 1;
}

sub _DeleteProfile {
    my ( $Obj ) = @_;
    $Obj->delete;
    return 1;
}

sub _InsertTestObject {
    my ( $Alias, $ClassName, $Parent ) = @_;
    my $Class = LoadClassByAlias( $ClassName );
    $Parent = LoadRootObject() unless $Parent;
    return $Class->insertObject({Alias=>$Alias,Parent=>$Parent});
}

sub _InsertTestObject_UOM {
    my ( $Alias, $Parent ) = @_;
    my $Class = LoadClassByAlias( 'UnitOfMeasurement' );
    $Parent = LoadRootObject() unless $Parent;
    return $Class->insertObject({Alias=>$Alias,Parent=>$Parent,Type=>'length',Factor=>0.34});
}

__PACKAGE__->runScript;
