#========================================================================================
# package      DE_EPAGES::Product::t::LastUpdateProfile
#----------------------------------------------------------------------------------------
# description  Test LastUpdate profiles for products
#========================================================================================
package DE_EPAGES::Product::t::LastUpdateProfile;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 20;

use DE_EPAGES::Core::API::PerlTools qw (AddDuration);
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Shop::API::Constants qw (TAX_MODEL_NET);

sub _initProfile {
    my $self = shift;
    my ($ProfileName) = @_;

    my $Profile = LoadClassByAlias('Product')->lastUpdateProfile($ProfileName);
    my $Shop = $self->shop;
    my $Now = $self->dbi->currentDateTime;

    $Shop->clearUpdatedObjects( $Profile, AddDuration($Now, 'seconds', 2) );
    my $Iterator = $Shop->getUpdatedObjects($Profile, $Now);
    is $Iterator->count, 0, "no $ProfileName updates yet";

    return $Profile;
}

sub testContent {
    my $self = shift;

    my $Profile = $self->_initProfile('Content');
    my $Shop = $self->shop;
    my $Now = $self->dbi->currentDateTime;
    my $EN = GetPKeyLanguageByCode('en');

    $Shop->path('Products/MyPetrol')->set({'Manufacturer' => 'Esso'});
    $Shop->path('Products/MyRedNose')->set({'Name' => 'My red nose'}, $EN );
    $Shop->path('Products/MyYellowNose')->set({'Text' => 'He has a yellow nose'}, $EN );
    $Shop->path('Products/Shirt')->set({'ImageSmall' => 'http://server/small.png'});
    $Shop->path('Products/Shirt/SubProducts/"Shirt red"')->set({'ImageMedium' => 'http://server/medium_red.png'});

    my $cAssertItemCount = sub {
        my ($Count) = @_;
        my $Iterator = $Shop->getUpdatedObjects($Profile, $Now);
        is $Iterator->count, $Count, 'content updates recorded' or do {
            while (my $hUpdate = $Iterator->next) {
               note 'Updated: (' . $hUpdate->{'Object'}->id . ') ' . $hUpdate->{'Object'}->get('RelativePath');
            }
        };
    };
    $cAssertItemCount->(5);

    $self->_initProfile('Content');
    $Shop->path('Products/MyPetrol')->set({'Manufacturer' => 'Esso'});
    $Shop->path('Products/MyRedNose')->set({'Name' => 'My red nose'}, $EN );
    $Shop->path('Products/MyYellowNose')->set({'Text' => 'He has a yellow nose'}, $EN );
    $Shop->path('Products/Shirt')->set({'ImageSmall' => 'http://server/small.png'});
    $Shop->path('Products/Shirt/SubProducts/"Shirt red"')->set({'ImageMedium' => 'http://server/medium_red.png'});
    $cAssertItemCount->(0);

    $Shop->path('Products/MyPetrol')->set({'Manufacturer' => undef});
    $cAssertItemCount->(1);
}

sub testListPrice {
    my $self = shift;

    my $Profile = $self->_initProfile('ListPrice');
    my $Shop = $self->shop;
    my $Now = $self->dbi->currentDateTime;
    my $Product = $Shop->path('Products/MyPetrol');

    $Product->setListPrice('EUR', TAX_MODEL_NET, 1.319); ## no critic qw(ProhibitMagicNumbers)

    my $Iterator = $Shop->getUpdatedObjects($Profile, $Now);
    is $Iterator->count, 1, 'one price update recorded';
    my $hUpdate = $Iterator->next;
    is $hUpdate->{'Object'}->id, $Product->id, 'updated object id';

    $self->_initProfile('ListPrice');
    $Product->setListPrice('EUR', TAX_MODEL_NET, 1.319); ## no critic qw(ProhibitMagicNumbers)

    $Iterator = $Shop->getUpdatedObjects($Profile, $Now);
    is $Iterator->count, 0, 'no identical price update recorded';
}

sub testStockLevel {
    my $self = shift;

    my $Profile = $self->_initProfile('StockLevel');
    my $Shop = $self->shop;
    my $Now = $self->dbi->currentDateTime;
    my $Product = $Shop->path('Products/MyPetrol');

    $Product->set({ StockLevel => 400 });

    my $Iterator = $Shop->getUpdatedObjects($Profile, $Now);
    is $Iterator->count, 1, 'one stock level update recorded';
    my $hUpdate = $Iterator->next;
    is $hUpdate->{'Object'}->id, $Product->id, 'updated object id';
}

sub testStockLevelOfBundle {
    my $self = shift;

    my $Profile = $self->_initProfile('StockLevel');
    my $Shop = $self->shop;
    my $Now = $self->dbi->currentDateTime;

    my $Cap = $Shop->path('Products/Cap');
    is $Cap->get('StockLevel'), 4, 'Cap.StockLevel before update';
    my $Bundle = $Shop->path('Products/Bundle');
    is $Bundle->get('StockLevel'), 2, 'Bundle.StockLevel before update';

    $Cap->set({ StockLevel => 6 });
    is $Bundle->get('StockLevel'), 3, 'Bundle.StockLevel after update';

    my $Iterator = $Shop->getUpdatedObjects($Profile, $Now);
    is $Iterator->count, 2, 'one update recorded';

    my %Updated;
    while( my $hUpdate = $Iterator->next ) {
        $Updated{$hUpdate->{'Object'}->id} = undef;
    }
    ok exists $Updated{$Cap->id}, 'updated bundled';
    ok exists $Updated{$Bundle->id}, 'updated bundle';
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

sub test {
    my $self = shift;

    $self->testContent;
    $self->testListPrice;
    $self->testStockLevel;
    $self->testStockLevelOfBundle;
}

__PACKAGE__->runScript;
